
local latest_instrument_idx, rns = 0

local options = renoise.Document.create {
  auto_checkmark = false
}
renoise.tool().preferences = options

function content()
  local content, added_patterntracks, order_in_track = { }, { }
  for track_idx, track in pairs(rns.tracks) do
    order_in_track = 0
    for pattern_idx, pattern in ipairs(rns.patterns) do
      if not added_patterntracks[pattern_idx .. "#" .. track_idx] then
        added_patterntracks[pattern_idx .. "#" .. track_idx] = true
        local patterntrack = pattern:track(track_idx)
        if not patterntrack.is_empty then
          order_in_track = order_in_track + 1
          table.insert(content, {
            ["track_idx"] = track_idx,
            ["sort"] = order_in_track + (track_idx/1000),
            ["patterntrack"] = patterntrack })
        end
      end
    end
  end
  table.sort(content, function(a, b) return a.sort < b.sort end )
  return content
end  

function find_track(instrument_idx)
  local content = content()
  local selected_instrument_idx = rns.selected_instrument_index - 1
  -- quick search (first line of all non-empty patterntracks)
  for _, val in ipairs(content) do
    local line = val.patterntrack:line(1)
    if not line.is_empty then
      for ncol_idx = 1, rns:track(val.track_idx).visible_note_columns do
        local ncol = line:note_column(ncol_idx)
        if not ncol.is_empty then
          if ncol.instrument_value == selected_instrument_idx then
            rns.selected_track_index = val.track_idx; return
          end
        end
      end
    end
  end
  -- full search (all lines in all non-empty patterntracks)
  for _, val in ipairs(content) do
    if not (selected_instrument_idx + 1 == latest_instrument_idx) then return end
    local patterntrack = val.patterntrack
    for _, line in ipairs(patterntrack.lines) do
      if not line.is_empty then
        for ncol_idx = 1, rns:track(val.track_idx).visible_note_columns do
          local ncol = line:note_column(ncol_idx)
          if not ncol.is_empty then
            if ncol.instrument_value == selected_instrument_idx then
              rns.selected_track_index = val.track_idx; return
            end
          end
        end
      end
    end
  end
end
  
function idle_check()
  if options.auto_checkmark.value then
    if not (rns.selected_instrument_index == latest_instrument_idx) then
      latest_instrument_idx = rns.selected_instrument_index
      local name = rns.selected_instrument.name
      if not (string.sub(name, 1, 2) == "--" or string.len(name) == 0) then
        find_track(latest_instrument_idx)
      end
    end
  end
end

function toggle_autocapture()
  options.auto_checkmark.value = not options.auto_checkmark.value
  autocapture_menu.selected = options.auto_checkmark.value
end

function initialize()
  rns = renoise.song()
  if options.auto_checkmark.value == true then
    latest_instrument_idx = rns.selected_instrument_index
    find_track(rns.selected_instrument_index)
  end
end

capture_menu = {
  name = "Instrument Box:Capture Track From Instrument",
  invoke = find_track,
  active = function() return not options.auto_checkmark.value end }
renoise.tool():add_menu_entry(capture_menu)

autocapture_menu = {
  name = "Instrument Box:Auto Capture Track From Instrument",
  invoke = toggle_autocapture,
  selected = function() return options.auto_checkmark.value end }
renoise.tool():add_menu_entry(autocapture_menu)

renoise.tool():add_keybinding {
  name = "Pattern Editor:Navigation:Capture track from instrument",
  invoke = find_track }

renoise.tool():add_keybinding {
  name = "Pattern Editor:Navigation:Toggle auto-capture track from instrument",
  invoke = toggle_autocapture }

renoise.tool().app_new_document_observable:add_notifier(initialize)
renoise.tool().app_idle_observable:add_notifier(idle_check)

